//
//  SpecificSettingViewController.m
//  scummvm
//
//  Created by Ilya Levin on 17/10/2015.
//
//

#import "SpecificSettingViewController.h"
#import "UIStoryboard+scummVMAdditions.h"
#import "ListDetailedCell.h"
#import "ListCell.h"
#import "SettingSliderCell.h"
#import "RadioListCell.h"
#import "NSStringAdditions.h"
#import "Strings.h"
#import "UIActivityViewController+MTAdditions.h"

using namespace std;
using namespace Enhanced;

@interface SpecificSettingViewController ()
{
    vector<const Enhanced::BaseSetting *> _setting;
    Enhanced::SettingSubMenuType _type;
}

@property (nonatomic) const vector<const Enhanced::BaseSetting *> setting;
@property (nonatomic) const Enhanced::SettingSubMenuType type;

@end

@implementation SpecificSettingViewController

-(void)setSetting:(const vector<const Enhanced::BaseSetting *>)setting
{
    _setting = setting;
}

-(void)setType:(Enhanced::SettingSubMenuType)type
{
    _type = type;
}

+(instancetype)settingViewConroller:(const vector<const Enhanced::BaseSetting *>) settings type:(Enhanced::SettingSubMenuType)type message:(NSString *)message
{
    SpecificSettingViewController* settingViewController = [[UIStoryboard mainStoryboard] instantiateViewControllerWithIdentifier:@"SpecificSettingViewController"];
    
    settingViewController.setting = settings;
    
    NSString* backTitle = [NSString stringWithCPPWString:Strings::getString(StringId::back)];

    settingViewController.negativeButton = backTitle;
    settingViewController.message = message;
    settingViewController.type = type;
    
    return settingViewController;
}

-(void)viewDidLoad
{
    [super viewDidLoad];
    
    self.tableView.allowsMultipleSelection = YES;
    [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(refreshSettings:) name:kRefreshSettingsNotification object:nil];
    
    self.tableView.alwaysBounceVertical = NO; //disable scrolling when not needed
}

- (void)dealloc
{
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

-(void)refreshSettings:(NSNotification*)notification
{
    [self.tableView reloadData];
}

-(void)negativeTapped:(id)sender
{
    
    self.nativeUIInterface->onSpecificSettingsFinished(self.type);
    [self.navigationController popViewControllerAnimated:NO];
}

-(void)indexPathTapped:(NSIndexPath *)indexPath
{
    // if radio group - select
    
}

#pragma mark - table view datasource
-(NSInteger)numberOfSectionsInTableView:(UITableView *)tableView
{
    unsigned long size = self.setting.size();
    
    return size;
}

-(NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section
{
    const Enhanced::BaseSetting* curSetting = (self.setting)[section];
    
    if (curSetting->getType() == Enhanced::SettingControlType::RADIO_GROUP) {
        const Enhanced::RadioGroupSetting* radioGroup = (const Enhanced::RadioGroupSetting*)curSetting;
        
        return radioGroup->getItemNames().size();
    }
    else return 1;
}

#pragma mark - table view delegate



-(CGFloat)tableView:(UITableView *)tableView heightForRowAtIndexPath:(NSIndexPath *)indexPath
{
    const Enhanced::BaseSetting* curSetting = (self.setting)[indexPath.section];
    if (curSetting->getType() == Enhanced::SettingControlType::RADIO_GROUP)
    {
        const Enhanced::RadioGroupSetting* radioGroup = (const Enhanced::RadioGroupSetting*)curSetting;

        if (radioGroup->getItemDescriptions().size() > 0)
        {
            return [self cellHeight]+25.0f;
        }
        else return [self cellHeight];
    }
    else
    {
        return [self cellHeight];
    }
}

-(UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath
{
    const Enhanced::BaseSetting* curSetting = (self.setting)[indexPath.section];
    
    if (curSetting->getType() == Enhanced::SettingControlType::RADIO_GROUP) {
        const Enhanced::RadioGroupSetting* radioGroup = (const Enhanced::RadioGroupSetting*)curSetting;
        
        NSString* title = [NSString stringWithCPPWString:radioGroup->getItemNames()[indexPath.row]];
        BOOL isEnabled = (radioGroup->getEnabledStates()[indexPath.row] == true);
        BOOL isSelected = radioGroup->getIntValue() == indexPath.row;
        
        if (isSelected) {
            [tableView selectRowAtIndexPath:indexPath animated:NO scrollPosition:UITableViewScrollPositionNone];
        }

        if (radioGroup->getItemDescriptions().size() > 0) {
            NSString* detail = [NSString stringWithCPPWString:radioGroup->getItemDescriptions()[indexPath.row]];
            ListDetailedCell* cell = [self.tableView dequeueReusableCellWithIdentifier:kListDetailedCellIdentifier forIndexPath:indexPath];

            [cell configureWithItemName:title detail:detail enabled:isEnabled];
            
            return cell;
        }
        else
        {
            RadioListCell* cell = [self.tableView dequeueReusableCellWithIdentifier:kRadioListCellIdentifier forIndexPath:indexPath];
            
            [cell configureWithTitle:title enabledState:isEnabled];
            
            return cell;
        }
    }
    else
    {
        SettingSliderCell* cell = [self.tableView dequeueReusableCellWithIdentifier:kSettingSliderCellIdentifier forIndexPath:indexPath];
        
        [cell configureWithSliderSetting:(const Enhanced::SliderSetting*) curSetting];
        
        return cell;
    }

}

-(BOOL)tableView:(UITableView *)tableView shouldHighlightRowAtIndexPath:(NSIndexPath *)indexPath
{
    const Enhanced::BaseSetting* curSetting = (self.setting)[indexPath.section];

    return (curSetting->getType() == Enhanced::SettingControlType::RADIO_GROUP);
}

-(NSIndexPath *)tableView:(UITableView *)tableView willSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    const Enhanced::BaseSetting* curSetting = (self.setting)[indexPath.section];
    
    return (curSetting->getType() == Enhanced::SettingControlType::RADIO_GROUP)?indexPath:nil;
}

-(NSIndexPath *)tableView:(UITableView *)tableView willDeselectRowAtIndexPath:(NSIndexPath *)indexPath
{
    BOOL otherRowSelectedInSection = NO;
    
    NSArray* selectedRows = [tableView indexPathsForSelectedRows];
    for (NSIndexPath* oldIndexPath in selectedRows) {
        if (![oldIndexPath isEqual:indexPath] && indexPath.section == oldIndexPath.section) {
            otherRowSelectedInSection = YES;
        }
    }
    
    return otherRowSelectedInSection? indexPath:nil;
}

-(void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    const Enhanced::BaseSetting* curSetting = (self.setting)[indexPath.section];

    NSArray* selectedRows = [tableView indexPathsForSelectedRows];
    for (NSIndexPath* oldIndexPath in selectedRows) {
        if (![oldIndexPath isEqual:indexPath] && indexPath.section == oldIndexPath.section) {
            [tableView deselectRowAtIndexPath:oldIndexPath animated:NO];
        }
    }
    
    curSetting->setIntValue((int)indexPath.row);
    // deselect any other rows in this radio group
}

-(IBAction)share:(id)sender
{
    UIActivityViewController* activityViewController = [UIActivityViewController shareMyAppActivityViewController];
    
    BOOL isIpad = [UIDevice currentDevice].userInterfaceIdiom == UIUserInterfaceIdiomPad;
    
    
    if (isIpad) {
        UIPopoverController* popoverController = [[UIPopoverController alloc] initWithContentViewController:activityViewController];
        
        [popoverController presentPopoverFromRect:[sender frame] inView:self.view permittedArrowDirections:UIPopoverArrowDirectionAny animated:YES];
    }
    else
    {
        [self presentViewController:activityViewController animated:YES completion:nil];
    }
}


@end
